<?php

declare(strict_types=1);

use App\Application\Controllers\MailController;
use App\Application\Actions\User\ListUsersAction;
use App\Application\Actions\User\ViewUserAction;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\App;
use Slim\Interfaces\RouteCollectorProxyInterface as Group;
use Slim\Routing\RouteCollectorProxy;

use Slim\Views\Twig;
use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception;
use PhpOffice\PhpSpreadsheet\IOFactory;

use Symfony\Component\Translation\Translator;
use Symfony\Component\Translation\Loader\ArrayLoader;

use GuzzleHttp\Client;


ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

return function (App $app) {
    $app->options('/{routes:.*}', function (Request $request, Response $response) {
        // CORS Pre-Flight OPTIONS Request Handler
        return $response;
    });

    // [ Functions ]
    function moveUploadedFile($directory, \Slim\Psr7\UploadedFile $uploadedFile)
    {
        $extension = pathinfo($uploadedFile->getClientFilename(), PATHINFO_EXTENSION);
        $basename = bin2hex(random_bytes(8)); // Generate a random filename
        $filename = sprintf('%s.%0.8s', $basename, $extension);
    
        $uploadedFile->moveTo($directory . DIRECTORY_SEPARATOR . $filename);
    
        return $filename;
    }

    // [ Base Route ]
    $app->get('/[{lang:(?:en|cn)}]', function (Request $request, Response $response) {
        $pdo = $this->get('db');

        if (!isset($_SESSION['user'])) {
            return $response
            ->withStatus(302) // HTTP status code for redirection
            ->withHeader('Location', '/login');
        }

        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);
        
        $mailSenders = $pdo->query("SELECT COUNT(*) FROM mail_senders")->fetchColumn();
        $mailSent = $pdo->query("SELECT COUNT(*) FROM mail_tasks WHERE mail_status = 'Success'")->fetchColumn();
        $mailPending = $pdo->query("SELECT COUNT(*) FROM mail_tasks WHERE mail_status = 'Pending'")->fetchColumn();
        $mailFailed = $pdo->query("SELECT COUNT(*) FROM mail_tasks WHERE mail_status = 'Failed'")->fetchColumn();
        
        $view = $this->get('view');

        return $view->render($response, 'home.twig', ['app' => ['lang' => $lang], 
            'mailSenders' => $mailSenders, 
            'mailSent' => $mailSent, 
            'mailPending' => $mailPending, 
            'mailFailed' => $mailFailed,
        ]);
    });

    // [ Language Route ]
    // Route to handle language switching
    $app->get('/set-language/{lang}', function ($request, $response, $args) {
        $lang = $args['lang'];
        
        // Set the language in session or cookie
        setcookie('lang', $lang, time() + (86400 * 30), "/"); // Cookie lasts for 30 days
        $_SESSION['lang'] = $lang;

        // Redirect back to the homepage or referer
        return $response->withHeader('Location', '/')->withStatus(302);
    });

    // [ Login Route ]
    $app->get('/login', function (Request $request, Response $response) {
        $pdo = $this->get('db');
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        // Fetch the Twig instance from the container
        $view = $this->get('view');
        return $view->render($response, 'login.twig', ['app' => ['lang' => $lang], 
            'login_message' => $translator->trans('login_message'),
            'login_username' => $translator->trans('login_username'),
            'login_password' => $translator->trans('login_password')
        ]);
    });

    // [ Mail Servers ]
    $app->get('/mailservers', function (Request $request, Response $response) {
        $pdo = $this->get('db');
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        // Fetch existing mail templates from the database
        $mailServers = $pdo->query("SELECT * FROM mail_servers")->fetchAll(PDO::FETCH_ASSOC);
    
        $view = $this->get('view');
        // Render the Twig view, passing in the templates
        return $view->render($response, 'mailservers.twig', [
            'app' => ['lang' => $lang],
            'mailServers' => $mailServers
        ]);
    });

    $app->get('/domainemails', function (Request $request, Response $response) {
        $pdo = $this->get('db');
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        $domain = $request->getQueryParams()['domain'];

        $stmt = $pdo->prepare("SELECT mail_address FROM mail_senders WHERE mail_address LIKE ?");
        $stmt->execute(['%' . $domain . '%']);
        $emailAccounts = $stmt->fetchAll(PDO::FETCH_ASSOC);

        $payload = json_encode($emailAccounts);
        $response->getBody()->write($payload);
        return $response->withHeader('Content-Type', 'application/json')->withStatus(200);
    });

    // [ Mail Templates ]
    $app->get('/mailtemplates', function (Request $request, Response $response) {
        $pdo = $this->get('db');
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        // Fetch existing mail templates from the database
        $mailTemplates = $pdo->query("SELECT * FROM mail_templates")->fetchAll(PDO::FETCH_ASSOC);
    
        $view = $this->get('view');
        // Render the Twig view, passing in the templates
        return $view->render($response, 'mailtemplates.twig', [
            'app' => ['lang' => $lang],
            'mailTemplates' => $mailTemplates
        ]);
    });

    // [ EMail Boxes ]
    $app->get('/mailboxes', function ($request, $response, $args) {
        $pdo = $this->get('db');
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        $stmt = $pdo->query("SELECT mail_address FROM mail_senders");
        $mailboxes = $stmt->fetchAll(PDO::FETCH_COLUMN);
    
        $view = $this->get('view');
        return $view->render($response, 'mailboxes.twig', ['app' => ['lang' => $lang], 'mailboxes' => $mailboxes]);
       
    });

    // [ Send Bulk Mail ]
    $app->get('/sendbulk', function ($request, $response, $args) {
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        $view = $this->get('view');
        return $view->render($response, 'sendmail.twig', ['app' => ['lang' => $lang]]);
       
    });

    // [ Mail Send Reports ]
    $app->get('/mailsendsuccess', function (Request $request, Response $response) {
        $pdo = $this->get('db');
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        // Fetch existing mail templates from the database
        $mailTemplates = $pdo->query("SELECT * FROM mail_tasks WHERE mail_status = 'Success'")->fetchAll(PDO::FETCH_ASSOC);
    
        // Render the Twig view, passing in the templates
        $view = $this->get('view');
        return $view->render($response, 'sendsuccess.twig', [
            'app' => ['lang' => $lang],
            'mailTemplates' => $mailTemplates
        ]);
    });

    $app->get('/mailsendfailed', function (Request $request, Response $response) {
        $pdo = $this->get('db');
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        // Fetch existing mail templates from the database
        $mailTemplates = $pdo->query("SELECT * FROM mail_tasks WHERE mail_status = 'Failed'")->fetchAll(PDO::FETCH_ASSOC);
    
        // Render the Twig view, passing in the templates
        $view = $this->get('view');
        return $view->render($response, 'sendfailed.twig', [
            'app' => ['lang' => $lang],
            'mailTemplates' => $mailTemplates
        ]);
    });

    $app->map(['GET','POST'], '/mailinbox', function (Request $request, Response $response, $args) {
        $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

        // Set the language for the translator
        $translator = $this->get(Translator::class);
        $translator->setLocale($lang);

        $mailbox = $request->getQueryParams()['mailbox'] ?? "zifezoimufi-5653@roti777.email";

        // Set up the IMAP connection
        $imapHost = '{premium176.web-hosting.com:993/imap/ssl}';
        $imapUsername = $mailbox;
        $imapPassword = 'Sy5adm!n01';

        $inbox = imap_open($imapHost, $imapUsername, $imapPassword);
        
        if (!$inbox) {
            $error = imap_last_error();
            return $response->withStatus(500)->write("Failed to connect to mailbox: $error");
        }

        function getPart($inbox, $emailNumber, $part, $partNumber) {
            $data = imap_fetchbody($inbox, $emailNumber, $partNumber);
            
            // Decode if necessary
            if ($part->encoding == 3) {
                // Base64
                $data = base64_decode($data);
            } elseif ($part->encoding == 4) {
                // Quoted-printable
                $data = quoted_printable_decode($data);
            }
            return $data;
        }
        
        // Recursive function to extract parts
        function getBody($inbox, $emailNumber, $structure, $partNumber = "") {
            $body = "";
            
            if ($structure->type == 1 && isset($structure->parts)) { // multipart
                foreach ($structure->parts as $index => $subPart) {
                    $subPartNumber = $partNumber ? $partNumber . "." . ($index + 1) : (string) ($index + 1);
                    if ($subPart->subtype == 'HTML') {
                        $body = getPart($inbox, $emailNumber, $subPart, $subPartNumber);
                    } elseif ($subPart->subtype == 'PLAIN' && !$body) {
                        // Fallback to plain text if no HTML is found
                        $body = getPart($inbox, $emailNumber, $subPart, $subPartNumber);
                    } elseif (isset($subPart->parts)) {
                        // Recursively go through nested parts
                        $body = getBody($inbox, $emailNumber, $subPart, $subPartNumber);
                    }
                }
            } else { // single part
                $body = getPart($inbox, $emailNumber, $structure, "1");
            }
            
            return $body;
        }

        // Fetch the latest 10 emails (adjust as needed)
        //$emails = imap_search($inbox, 'ALL');
        $emails = imap_search($inbox, 'SUBJECT "Mail delivery failed"');
        
        $emailData = [];
        
        if ($emails) {
            // Sort emails by latest first
            rsort($emails);

            // Get details of each email (for example, first 10)
            foreach (array_slice($emails, 0, 10) as $emailNumber) {
                $header = imap_headerinfo($inbox, $emailNumber);
                $overview = imap_fetch_overview($inbox, (string) $emailNumber, 0);
                $structure = imap_fetchstructure($inbox, $emailNumber);
    
                $body = '';
                
                // Get the body of the email
                $body = getBody($inbox, $emailNumber, $structure);
                
                $emailData[] = [
                    'subject' => $overview[0]->subject,
                    'from' => $overview[0]->from,
                    'date' => $overview[0]->date,
                    'body' => $body
                ];
            }
        }

        // Close the IMAP connection
        imap_close($inbox);

        // Render inbox.twig and pass the email data
        $view = $this->get('view');
        return $view->render($response, 'inbox-partial.twig', [
            'app' => ['lang' => $lang],
            'mailbox' => $mailbox,
            'emails' => $emailData,
        ]);

        
    });   

    // [ API Routes ]
    // API Routes
    $app->group('/api', function ($group) {
        
        $group->map(['GET','POST'], '/login', function (Request $request, Response $response, $args)  {
            $pdo = $this->get('db');
            $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

            // Set the language for the translator
            $translator = $this->get(Translator::class);
            $translator->setLocale($lang);
        
            $parsedBody = $request->getParsedBody();
            $password = $parsedBody['password'] ?? '';
            $username = $parsedBody['username'] ?? '';
            
            $userData = $pdo->prepare("SELECT * FROM mail_users WHERE user_name = :user and user_pass = :pass");
            $userData->bindParam(':user', $username);
            $userData->bindParam(':pass', $password);
            $userData->execute();
            $userResult = $userData->fetchAll(PDO::FETCH_ASSOC);
            $rows = $userData->rowCount();
            
            if ($userResult && $rows > 0)
            {
                $_SESSION['user'] = 'admin';
                return $response->withHeader('Location', '/')->withStatus(302);
            } else {
                return $response->withHeader('Location', '/login')->withStatus(302);
            }
        
        });
    });

    $app->group('/mail', function (RouteCollectorProxy $group)  {
 
        $group->map(['GET','POST'], '/addtemplate', function (Request $request, Response $response, $args) {
            $pdo = $this->get('db');
            $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

            $parsedBody = $request->getParsedBody();
        
            // Input Validation
            if (empty($parsedBody['mail_title']) || empty($parsedBody['mail_message'])) {
                $result = ['success' => false, 'message' => 'Invalid input'];
            }
            else
            {
                // Validate and update the database
                $mail_title = $parsedBody['mail_title'];
                $mail_message = $parsedBody['mail_message'];
            
                // Perform the update operation in the database
                $sql = $pdo->prepare("INSERT INTO mail_templates (mail_title, mail_message) VALUES (:mail_title, :mail_message)");
                $sql->bindParam(':mail_title', $mail_title);
                $sql->bindParam(':mail_message', $mail_message);
                $result = $sql->execute();
            
                if ($result) {
                    $results = ['success' => true];
                } else {
                    $results = ['success' => false, 'message' => 'Update failed'];
                }
            }
        
            $payload = json_encode($results);
            $response->getBody()->write($payload);
            return $response->withHeader('Content-Type', 'application/json')->withStatus(200);
        });

        $group->map(['GET','POST'], '/updatetemplate', function (Request $request, Response $response, $args) {
            $pdo = $this->get('db');
            $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

            $parsedBody = $request->getParsedBody();
        
            // Input Validation
            if (empty($parsedBody['mail_id']) || empty($parsedBody['mail_title']) || empty($parsedBody['mail_message'])) {
                $results = ['success' => false, 'message' => 'Invalid input'];
            }
            else
            {
                // Validate and update the database
                $mail_id = $parsedBody['mail_id'];
                $mail_title = $parsedBody['mail_title'];
                $mail_message = $parsedBody['mail_message'];
            
                // Perform the update operation in the database
                $sql = $pdo->prepare("UPDATE mail_templates SET mail_title = :mail_title, mail_message = :mail_message WHERE mail_template_id = :id");
                $sql->bindParam(':mail_title', $mail_title);
                $sql->bindParam(':mail_message', $mail_message);
                $sql->bindParam(':id', $mail_id);
                $result = $sql->execute();
            
                if ($result) {
                    $results = ['success' => true];
                } else {
                    $results = ['success' => false, 'message' => 'Update failed'];
                }
            }
        
            $payload = json_encode($results);
            $response->getBody()->write($payload);
            return $response->withHeader('Content-Type', 'application/json')->withStatus(200);
        });
        
        $group->post('/upload', function (Request $request, Response $response, $args) {
            $pdo = $this->get('db');
            $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';

            $uploadedFiles = $request->getUploadedFiles();
            $file = $uploadedFiles['file'];
        
            // Ensure file is XLSX
            if ($file->getClientMediaType() !== 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet') {
                return $response->withStatus(400)->write('Invalid file type');
            }
        
            // Save uploaded file to temp directory
            $uploadDir = __DIR__ . '/../public/uploads/';
            $filename = moveUploadedFile($uploadDir, $file);
        
            // Process file and save data to DB
            $spreadsheet = IOFactory::load($uploadDir . $filename);
            $data = $spreadsheet->getActiveSheet()->toArray();
        
            // Get all available mail addresses
            $mailAddresses = $pdo->query("SELECT mail_address FROM mail_senders")->fetchAll(PDO::FETCH_ASSOC);
            
            if (empty($mailAddresses)) {
                return $response->withStatus(400)->write('No available mail senders found');
            }
        
            // Shuffle the mail addresses for randomness
            shuffle($mailAddresses);
        
            // Track used addresses for this task session
            $usedMailAddresses = [];
            $currentIndex = 0;
        
            // Function to get the next available mail sender (round-robin from shuffled list)
            function getNextMailSender(&$mailAddresses, &$usedMailAddresses, &$currentIndex)
            {
                if ($currentIndex >= count($mailAddresses)) {
                    // If all mail addresses are used, shuffle and reset index
                    shuffle($mailAddresses);
                    $usedMailAddresses = [];  // Reset the used addresses
                    $currentIndex = 0;        // Start from the first one again
                }
        
                // Get the current mail address
                $mailSender = $mailAddresses[$currentIndex];
                $usedMailAddresses[] = $mailSender['mail_address'];
        
                // Move to the next one in the list
                $currentIndex++;
        
                return $mailSender['mail_address'];
            }
        
            $insertedData = [];
        
            foreach ($data as $index => $row) {
                if ($index === 0) {
                    continue; // Skip header row
                }
        
                // Get the next available mail sender from shuffled list (random + round-robin)
                $mailReply = getNextMailSender($mailAddresses, $usedMailAddresses, $currentIndex);
        
                // Save each row to the database
                $stmt = $pdo->prepare("INSERT INTO mail_tasks (mail_reply, mail_email, mail_var1, mail_var2, mail_var3, mail_status)
                                    VALUES (?, ?, ?, ?, ?, 'Pending')");
        
                // Bind the values and execute the statement
                $stmt->execute([
                    $mailReply,
                    $row[0], // mail_email
                    $row[1], // mail_var1
                    $row[2], // mail_var2
                    $row[3]  // mail_var3
                ]);
        
                $insertedData[] = [
                    'mail_reply' => $mailReply,
                    'mail_email' => $row[0],
                    'mail_var1' => $row[1],
                    'mail_var2' => $row[2],
                    'mail_var3' => $row[3],
                    'mail_status' => 'Pending'
                ];
            }
        
            // Respond with success and progress bar overlay
            $payload = json_encode(['success' => true, 'message' => 'File uploaded successfully', 'data' => $insertedData]);
            $response->getBody()->write($payload);
        
            return $response->withHeader('Content-Type', 'application/json');
        });

        $group->map(['GET','POST'], '/process', function (Request $request, Response $response, $args) {
            
            ob_start();
             
            $pdo = $this->get('db');
            $lang = $_SESSION['lang'] ?? $_COOKIE['lang'] ?? 'cn';
        
            // Check if the process is currently running
            $lockStmt = $pdo->query("SELECT is_running FROM mail_process_status ORDER BY id DESC LIMIT 1");
            $isRunning = $lockStmt->fetchColumn();
        
            if ($isRunning) {
                // If the process is already running, send a response and exit
                $payload = json_encode([
                    'message' => 'Process is already running. Please wait until it completes.'
                ]);
                $response->getBody()->write($payload);
                return $response->withHeader('Content-Type', 'application/json')->withStatus(200);
            }
        
            // Set the process as running
            $pdo->query("UPDATE mail_process_status SET is_running = 1 WHERE id = (SELECT id FROM mail_process_status ORDER BY id DESC LIMIT 1)");
        
            // Output a response to the browser immediately
            $payload = json_encode([
                'message' => 'Processing started. Check back later for results.'
            ]);
        
            // Send the response and finish the request early
            $response->getBody()->write($payload);
            $response = $response->withHeader('Content-Type', 'application/json')->withStatus(200);
            
            // Flush the buffer and end the client connection
            echo $payload;
            ob_end_flush();
            flush();
        
            // Proceed with email sending logic
            try {
                // Fetch the last execution time
                $lastExecutionStmt = $pdo->query("SELECT last_execution_time FROM mail_process_status ORDER BY id DESC LIMIT 1");
                $lastExecutionTime = $lastExecutionStmt->fetchColumn();
        
                $currentTime = new DateTime();
                if ($lastExecutionTime) {
                    $lastExecutionTime = new DateTime($lastExecutionTime);
                    $interval = $currentTime->getTimestamp() - $lastExecutionTime->getTimestamp();
        
                    // If it's been less than 2 hours (7200 seconds), exit the process
                    if ($interval < 7200) {
                        return;
                    }
                }
        
                // Fetch all mail servers with their domain and sending limit
                $serverStmt = $pdo->query("SELECT mail_domain, mail_limit FROM mail_servers");
                $mailServers = $serverStmt->fetchAll(PDO::FETCH_ASSOC);
        
                $results = []; // To store the results of email sending
                $lastResponse = false;
        
                if ($mailServers) {
                    foreach ($mailServers as $mailServer) {
                        $mailDomain = $mailServer['mail_domain'];
                        $mailLimit = (int) $mailServer['mail_limit'];
        
                        // Fetch pending emails for the current domain
                        $stmt = $pdo->prepare("SELECT mail_id, mail_email, mail_reply, mail_var1, mail_var2, mail_var3 
                                               FROM mail_tasks 
                                               WHERE mail_status = 'Pending' AND mail_reply LIKE :mailDomain
                                               LIMIT :mailLimit");
                        $stmt->bindValue(':mailDomain', '%' . $mailDomain);
                        $stmt->bindValue(':mailLimit', $mailLimit, PDO::PARAM_INT);
                        $stmt->execute();
                        $pendingEmails = $stmt->fetchAll(PDO::FETCH_ASSOC);
        
                        if ($pendingEmails) {
                            $sentCount = 0;
        
                            foreach ($pendingEmails as $emailTask) {
                                $mail = new PHPMailer(true);
                                $mail->isSMTP();
                                $mail->SMTPOptions = [
                                    'ssl' => [
                                        'verify_peer' => false,
                                        'verify_peer_name' => false,
                                        'allow_self_signed' => true
                                    ]
                                ];
                                $mail->Host = 'premium176.web-hosting.com';
                                $mail->SMTPAuth = true;
                                $mail->Username = $emailTask['mail_reply'];
                                $mail->Password = 'Sy5adm!n01';
                                $mail->SMTPSecure = PHPMailer::ENCRYPTION_STARTTLS;
                                $mail->Port = 587;
        
                                // Fetch random email content
                                $content = getRandomMailContent($pdo);
                                $personalizedHTMLMessage = str_replace(
                                    ['{{var1}}', '{{var2}}', '{{var3}}'],
                                    [$emailTask['mail_var1'], $emailTask['mail_var2'], $emailTask['mail_var3']],
                                    $content['emailMessage']
                                );
        
                                $subject = str_replace(
                                    ['{{var1}}', '{{var2}}', '{{var3}}'],
                                    [$emailTask['mail_var1'], $emailTask['mail_var2'], $emailTask['mail_var3']],
                                    $content['emailTitle']
                                );
        
                                try {
                                    $mail->clearAddresses();
                                    $mail->addAddress($emailTask['mail_email']);
                                    $mail->setFrom($emailTask['mail_reply'], "ROTI777 Marketing");
                                    $mail->addReplyTo($emailTask['mail_reply'], "ROTI777 Marketing");
                                    $mail->Subject = $subject;
                                    $mail->isHTML(true);
                                    $mail->Body = $personalizedHTMLMessage;
        
                                    // Send the email
                                    if ($mail->send()) {
                                        // Update the status to 'Success'
                                        $stmt = $pdo->prepare("UPDATE mail_tasks SET mail_status = 'Success', mail_title = ?, mail_content = ? WHERE mail_id = ?");
                                        $stmt->execute([$subject, $personalizedHTMLMessage, $emailTask['mail_id']]);
                                        $results[] = "Email sent successfully to: {$emailTask['mail_email']}";
                                    } else {
                                        // Failed to send, update the reason
                                        $stmt = $pdo->prepare("UPDATE mail_tasks SET mail_status = 'Failed', mail_reason = ? WHERE mail_id = ?");
                                        $stmt->execute([$mail->ErrorInfo, $emailTask['mail_id']]);
                                        $results[] = "Failed to send email to: {$emailTask['mail_email']}. Error: {$mail->ErrorInfo}";
                                        continue;
                                    }
        
                                    // Increment the sent email count
                                    $sentCount++;
                                    if ($sentCount >= $mailLimit) {
                                        // If the domain's limit has been reached, break out of the loop
                                        break;
                                    }
        
                                } catch (Exception $e) {
                                    // Update the status to 'Failed'
                                    $stmt = $pdo->prepare("UPDATE mail_tasks SET mail_status = 'Failed', mail_reason = ? WHERE mail_id = ?");
                                    $stmt->execute([$mail->ErrorInfo, $emailTask['mail_id']]);
                                    $results[] = "Failed to send email to: {$emailTask['mail_email']}. Error: {$mail->ErrorInfo}";
                                    continue;
                                }
        
                                sleep(rand(1, 3)); // To avoid overloading the mail server
                            }
                        } else {
                            $results[] = "No pending emails to send for domain: {$mailDomain}.";
                        }
                    }
                } else {
                    $results[] = "No mail servers configured.";
                }
        
                if (count($results) > 0) {
                    $lastResponse = true; // Set this to true if there are results
                }
        
                // Store the current time as the last execution time
                $stmt = $pdo->prepare("INSERT INTO mail_process_status (last_execution_time, is_running) VALUES (NOW(), 0)");
                $stmt->execute();
        
            } finally {
                // Set the process as not running
                $pdo->query("UPDATE mail_process_status SET is_running = 0 WHERE id = (SELECT id FROM mail_process_status ORDER BY id DESC LIMIT 1)");
            }
        
            return $response;
        }); 
        
        // Fetch random email content from the database
        function getRandomMailContent($pdo)
        {
            $mailStmt = $pdo->query("SELECT * FROM mail_templates ORDER BY RAND() LIMIT 1");
            $mailData = $mailStmt->fetch(PDO::FETCH_ASSOC);

            return [
                'emailTitle'   => $mailData['mail_title'],
                'emailMessage' => $mailData['mail_message']
            ];
        }
    });
};
